@@ -39,29 +39,15 @@
 #include <QtCore/qmath.h>
 
 #define PHONON_CLASSNAME AudioOutput
+#define IFECES7 AudioOutputInterface47
 #define IFACES2 AudioOutputInterface42
 #define IFACES1 IFACES2
-#define IFACES0 AudioOutputInterface40, IFACES1
+#define IFACES0 AudioOutputInterface40, IFACES1, IFECES7
 #define PHONON_INTERFACENAME IFACES0
 
-QT_BEGIN_NAMESPACE
-
 namespace Phonon
 {
 
-static inline bool callSetOutputDevice(AudioOutputPrivate *const d, int index)
-{
-    PulseSupport *pulse = PulseSupport::getInstance();
-    if (pulse->isActive())
-        return pulse->setOutputDevice(d->getStreamUuid(), index);
-
-    Iface<IFACES2> iface(d);
-    if (iface) {
-        return iface->setOutputDevice(AudioOutputDevice::fromIndex(index));
-    }
-    return Iface<IFACES0>::cast(d)->setOutputDevice(index);
-}
-
 static inline bool callSetOutputDevice(AudioOutputPrivate *const d, const AudioOutputDevice &dev)
 {
     PulseSupport *pulse = PulseSupport::getInstance();
@@ -118,6 +104,10 @@ void AudioOutputPrivate::init(Phonon::Category c)
             q->connect(stream, SIGNAL(usingDevice(int)), SLOT(_k_deviceChanged(int)));
             q->connect(stream, SIGNAL(volumeChanged(qreal)), SLOT(_k_volumeChanged(qreal)));
             q->connect(stream, SIGNAL(muteChanged(bool)), SLOT(_k_mutedChanged(bool)));
+
+            AudioOutputInterface47 *iface = Iface<AudioOutputInterface47>::cast(this);
+            if (iface)
+                iface->setStreamUuid(streamUuid);
         }
     }
 #endif
@@ -285,7 +275,7 @@ bool AudioOutput::setOutputDevice(const AudioOutputDevice &newAudioOutputDevice)
         d->device = newAudioOutputDevice;
     }
     if (k_ptr->backendObject()) {
-        return callSetOutputDevice(d, d->device.index());
+        return callSetOutputDevice(d, d->device);
     }
     return true;
 }
@@ -550,12 +540,11 @@ AudioOutputPrivate::~AudioOutputPrivate()
 
 } //namespace Phonon
 
-QT_END_NAMESPACE
-
 #include "moc_audiooutput.cpp"
 
 #undef PHONON_CLASSNAME
 #undef PHONON_INTERFACENAME
+#undef IFECES7
 #undef IFACES2
 #undef IFACES1
 #undef IFACES0
