@@ -1,5 +1,6 @@
-/*  This file is part of the KDE project
-    Copyright (C) 2009 Colin Guthrie <cguthrie@mandriva.org>
+/*
+    Copyright (C) 2010 Colin Guthrie <cguthrie@mandriva.org>
+    Copyright (C) 2013 Harald Sitter <sitter@kde.org>
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Lesser General Public
@@ -17,7 +18,6 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library.  If not, see <http://www.gnu.org/licenses/>.
-
 */
 
 #include "pulsesupport.h"
@@ -27,6 +27,7 @@
 #include <QtCore/QStringList>
 #include <QtCore/QTimer>
 #include <QtCore/QMutex>
+#include <QApplication>
 
 #ifdef HAVE_PULSEAUDIO
 #include "pulsestream_p.h"
@@ -41,8 +42,9 @@
 #endif // HAVE_PULSEAUDIO
 
 #include "phononnamespace_p.h"
+#include "platform_p.h"
 
-QT_BEGIN_NAMESPACE
+#define PA_PROP_PHONON_STREAMID "phonon.streamid"
 
 namespace Phonon
 {
@@ -541,7 +543,7 @@ static void sink_input_cb(pa_context *c, const pa_sink_input_info *i, int eol, v
 
     // loop through (*i) and extract phonon->streamindex...
     const char *t;
-    if ((t = pa_proplist_gets(i->proplist, "phonon.streamid"))) {
+    if ((t = pa_proplist_gets(i->proplist, PA_PROP_PHONON_STREAMID))) {
         logMessage(QString::fromLatin1("Found PulseAudio stream index %1 for Phonon Output Stream %2").arg(i->index).arg(QLatin1String(t)));
 
         // We only care about our own streams (other phonon processes are irrelevent)
@@ -584,7 +586,7 @@ static void source_output_cb(pa_context *c, const pa_source_output_info *i, int
 
     // loop through (*i) and extract phonon->streamindex...
     const char *t;
-    if ((t = pa_proplist_gets(i->proplist, "phonon.streamid"))) {
+    if ((t = pa_proplist_gets(i->proplist, PA_PROP_PHONON_STREAMID))) {
         logMessage(QString::fromLatin1("Found PulseAudio stream index %1 for Phonon Capture Stream %2").arg(i->index).arg(QLatin1String(t)));
 
         // We only care about our own streams (other phonon processes are irrelevent)
@@ -818,6 +820,12 @@ PulseSupport::PulseSupport()
         return;
     }
 
+    if (!QAbstractEventDispatcher::instance() || !QAbstractEventDispatcher::instance()->metaObject()) {
+        qWarning("WARNING: Cannot construct PulseSupport because there is no Eventloop."
+                 " May be because of application shutdown.");
+        return;
+    }
+
     // We require a glib event loop
     if (!QByteArray(QAbstractEventDispatcher::instance()->metaObject()->className()).contains("EventDispatcherGlib")) {
         qWarning("WARNING: Disabling PulseAudio integration for lack of GLib event loop.");
@@ -1118,13 +1126,22 @@ void PulseSupport::setCaptureDevicePriorityForCategory(Category category, QList<
 static PulseStream* register_stream(QMap<QString,PulseStream*> &map, QString streamUuid, QString role)
 {
     logMessage(QString::fromLatin1("Initialising streamindex %1").arg(streamUuid));
-    if (!role.isEmpty()) {
-        logMessage(QString::fromLatin1("Setting role to %1 for streamindex %2").arg(role).arg(streamUuid));
-        setenv("PULSE_PROP_media.role", role.toLatin1().constData(), 1);
-    }
-    setenv("PULSE_PROP_phonon.streamid", streamUuid.toLatin1().constData(), 1);
-    PulseStream *stream = new PulseStream(streamUuid);
+
+    PulseStream *stream = new PulseStream(streamUuid, role);
     map[streamUuid] = stream;
+
+    // Setup envrionment...
+    // These values are considered static, so we force property overrides for them.
+    if (!Platform::applicationName().isEmpty())
+        qputenv(QString("PULSE_PROP_OVERRIDE_%1").arg(PA_PROP_APPLICATION_NAME).toUtf8(),
+                Platform::applicationName().toUtf8());
+    if (!qApp->applicationVersion().isEmpty())
+        qputenv(QString("PULSE_PROP_OVERRIDE_%1").arg(PA_PROP_APPLICATION_VERSION).toUtf8(),
+                qApp->applicationVersion().toUtf8());
+    if (!qApp->applicationName().isEmpty())
+        qputenv(QString("PULSE_PROP_OVERRIDE_%1").arg(PA_PROP_APPLICATION_ICON_NAME).toUtf8(),
+                qApp->applicationName().toUtf8());
+
     return stream;
 }
 
@@ -1175,6 +1192,59 @@ PulseStream *PulseSupport::registerCaptureStream(QString streamUuid, Category ca
 #endif
 }
 
+QHash<QString, QString> PulseSupport::streamProperties(QString streamUuid) const
+{
+    QHash<QString, QString> properties;
+
+#ifdef HAVE_PULSEAUDIO
+    PulseStream *stream = 0;
+
+    // Try to find the stream among the known output streams.
+    if (!stream)
+        stream = s_outputStreams.value(streamUuid);
+
+    // Not an output stream, try capture streams.
+    if (!stream)
+        stream = s_captureStreams.value(streamUuid);
+
+    // Also no capture stream, start crying and return an empty hash.
+    if (!stream) {
+        qWarning() << Q_FUNC_INFO << "Requested UUID Could not be found. Returning with empty properties.";
+        return properties;
+    }
+
+    properties[QLatin1String(PA_PROP_PHONON_STREAMID)] = stream->uuid();
+    properties[QLatin1String(PA_PROP_MEDIA_ROLE)] = stream->role();
+
+    // Tear down environment before returning. This is to prevent backends from
+    // being overridden by the environment if present.
+    QHashIterator<QString, QString> it(properties);
+    while (it.hasNext()) {
+        it.next();
+        unsetenv(QString("PULSE_PROP_OVERRIDE_%1").arg(it.key()).toUtf8());
+    }
+#endif // HAVE_PULSEAUDIO
+
+    return properties;
+}
+
+void PulseSupport::setupStreamEnvironment(QString streamUuid)
+{
+    pDebug() << "Please note that your current Phonon backend is trying to force"
+                " stream dependent PulseAudio properties through envrionment variables."
+                " Slightly unprecise timing in doing so will prevent the first"
+                " of two subsequently started AudioOutputs to have disfunct volume"
+                " control. Also see https://bugs.kde.org/show_bug.cgi?id=321288";
+
+    const QHash<QString, QString> properties = streamProperties(streamUuid);
+
+    QHashIterator<QString, QString> it(properties);
+    while (it.hasNext()) {
+        it.next();
+        qputenv(QString("PULSE_PROP_OVERRIDE_%1").arg(it.key()).toUtf8(), it.value().toUtf8());
+    }
+}
+
 void PulseSupport::emitObjectDescriptionChanged(ObjectDescriptionType type)
 {
     if (mEnabled)
@@ -1261,6 +1331,10 @@ bool PulseSupport::setOutputVolume(QString streamUuid, qreal volume) {
             return false;
         }
         pa_operation_unref(o);
+    } else if (s_outputStreams.contains(streamUuid) && s_outputStreams[streamUuid]->index() == PA_INVALID_INDEX) {
+        logMessage(QString::fromLatin1("Setting volume on an invalid stream ..... this better be intended"));
+        PulseStream *stream = s_outputStreams[streamUuid];
+        stream->setCachedVolume(volume);
     }
     return true;
 #endif
@@ -1349,8 +1423,4 @@ void PulseSupport::clearStreamCache(QString streamUuid) {
 
 } // namespace Phonon
 
-QT_END_NAMESPACE
-
 #include "moc_pulsesupport.cpp"
-
-// vim: sw=4 ts=4
